;; minibuffer-test.el - Test functions in minibuffer-patches.el -*- lexical-binding:t -*-

(require 'minibuffer-patches)

(defun find-absfile-create-test-files ()
  "Create test dirs and files; return path."
  (interactive)
  (let* ((root (make-temp-file "find-file-path-test" t))
	 (dir1 (concat root "/dir1"))
	 (dir2 (concat root "/dir2")))

    (mkdir dir1)
    (mkdir dir2)

    (with-temp-file (concat dir1 "/file1.text")
      (insert "dir1/file1.text"))
    (with-temp-file (concat dir1 "/file2.text")
      (insert "dir1/file2.text"))

    (with-temp-file (concat dir2 "/file1.text")
      (insert "dir2/file1.text"))
    (with-temp-file (concat dir2 "/file3.text")
      (insert "dir2/file3.text"))

    (list (file-name-as-directory dir1)
	  (file-name-as-directory dir2))))

(defvar test-path (find-absfile-create-test-files))
(defvar dir1 (nth 0 test-path))
(defvar dir2 (nth 1 test-path))

(ert-deftest test-find-absfile-path-completion-table ()
  "Test basic functions of table."
  ;; grouped by action
  (should (equal (find-absfile-completion-table test-path "fi" nil '(boundaries ".text"))
		   '(boundaries . (0 . 2))))

  (should (equal (find-absfile-completion-table test-path "fi" nil 'metadata)
		 (cons 'metadata
		       (list
			'(category . absfile)
			(cons 'path test-path)))))

  ;; all-completions
  (should (equal (find-absfile-completion-table test-path "fi" nil t)
		 (list
		  (concat dir1 "file2.text")
		  (concat dir1 "file1.text")
		  (concat dir2 "file3.text")
		  (concat dir2 "file1.text")
		  )))

  (should (equal (find-absfile-completion-table test-path "file1.text<dir1/>" nil t)
		 ;; some caller did not deuniquify; treated as no match
		 nil))


  ;; try-completion
  (should (equal (find-absfile-completion-table test-path "fi" nil nil)
		 "file"))

  (should (equal (find-absfile-completion-table test-path "fil1" nil nil) ;; misspelled
		 nil))

  (should (equal (find-absfile-completion-table test-path "file2" nil nil) ;; unique but not exact
		 "file2.text"))

  (should (equal (find-absfile-completion-table test-path "file2.text" nil nil) ;; unique and exact
		 t))

  (should (equal (find-absfile-completion-table test-path "file1.text<dir1>" nil nil)
		 ;; Some caller did not deuniquify; treated as no match.
		 nil))

  ;; test-completion
  (should (equal (find-absfile-completion-table test-path "fi" nil 'lambda) ;; not unique
		 nil))

  (should (equal (find-absfile-completion-table test-path "file1.text<dir1/>" nil 'lambda);; unique and exact
		 ;; Some caller did not deuniquify: treated as no match.
		 nil))

  (should (equal (find-absfile-completion-table test-path (concat dir1 "file2.text") nil 'lambda);; unique and exact
		 t))
  )

(ert-deftest test-completion-test-completion ()
  (let ((find-absfile-uniquify-style 'abbrev)
	(table (apply-partially 'find-absfile-completion-table test-path)))

    (should (equal (test-completion "fi" table) ;; not unique
		 nil))

    (should (equal (test-completion "file1.text<dir1/>" table);; unique and exact
		 t))

    (should (equal (test-completion "file2.text" table test-path);; unique and exact
		 t))
  ))

(ert-deftest test-completion-get-data-string ()
  (let ((find-absfile-uniquify-style 'abbrev)
	(table (apply-partially 'find-absfile-completion-table test-path)))

    (should (equal (completion-get-data-string "file1.text<dir1/>" table nil)
		   (concat dir1 "file1.text")))

    (should (equal (completion-get-data-string "file2.text" table nil)
		   (concat dir1 "file2.text")))

    (should (equal (completion-get-data-string "file3.text" table nil)
		   (concat dir2 "file3.text")))
    ))

(ert-deftest test-completion-uniquify-absfile-all-completions ()
  (let ((find-absfile-uniquify-style 'abbrev)
	(table (apply-partially 'find-absfile-completion-table test-path))
	string)

    (setq string "fi")
    (should (equal (completion-uniquify-absfile-all-completions string table nil (length string))
		   (list "file1.text<dir1/>" "file1.text<dir2/>" "file2.text" "file3.text")))

    (setq string "file2")
    (should (equal (completion-uniquify-absfile-all-completions string table nil (length string))
		   (list "file2.text")))
    ))

(ert-deftest test-completion-uniquify-absfile-try-completion ()
  (let ((find-absfile-uniquify-style 'abbrev)
	(table (apply-partially 'find-absfile-completion-table test-path))
	string)

    (setq string "fi")
    (should (equal (completion-uniquify-absfile-try-completion string table nil (length string))
		   '("file" . 4)))

    (setq string "file2") ;; unique but not exact
    (should (equal (completion-uniquify-absfile-try-completion string table nil (length string))
		   '("file2.text" . 10)))

    (setq string "file2.text") ;; unique and exact
    (should (equal (completion-uniquify-absfile-try-completion string table nil (length string))
		   t))

    (setq string "fil2") ;; misspelled
    (should (equal (completion-uniquify-absfile-try-completion string table nil (length string))
		   nil))
    ))

(ert-deftest test-find-absfile-deuniquify ()

  (should (equal (find-absfile-deuniquify test-path "file1.text")
		 "file1.text"))

  (should (equal (find-absfile-deuniquify test-path "file2.text")
		 "file2.text"))

  (should (equal (find-absfile-deuniquify test-path "file2.text" t) ;; unique
		 (concat dir1 "file2.text")))

  (should (equal (find-absfile-deuniquify test-path "file3.text")
		 "file3.text"))

  (should (equal (find-absfile-deuniquify test-path "file3.text" t) ;; unique
		 (concat dir2 "file3.text")))

  (should (equal (find-absfile-deuniquify test-path "file1<dir2/>") ;; not exact
		 (concat dir2 "file1")))

  (should (equal (find-absfile-deuniquify test-path "file1.text<dir2/>") ;; exact
		 (concat dir2 "file1.text")))
  )

(ert-deftest test-find-absfile-uniquify ()
  (let ((find-absfile-uniquify-style 'abbrev))

    (should (equal (find-absfile-uniquify
		    '("/dir1/file1.text" "/dir1/file2.text" "/dir2/file1.text" "/dir2/file3.text"))
		   (list "file1.text<dir1/>" "file1.text<dir2/>" "file2.text" "file3.text")))

    (should (equal (find-absfile-uniquify '("/dir1/file1.text" "/dir2/file1.text"))
		   (list "file1.text<dir1/>" "file1.text<dir2/>")))

    (should (equal (find-absfile-uniquify '("/dir1/file2.text"))
		   (list "file2.text")))

    (should (equal (find-absfile-uniquify
		    '("/dir1/file1.text" "/dir1/file2.text" "/dir2/file1.text" "/dir2/file3.text"))
		   (list "file1.text<dir1/>" "file1.text<dir2/>" "file2.text" "file3.text")))
    ))

(provide 'test/minibuffer-test)
